#=====================================================================
# Standardowy (poredni ) model PLSP
#=====================================================================

#=====================================================================
param Model symbolic := "PLSP";
param Dane symbolic;
param Wyniki symbolic := Model&"_"&Dane&".csv";
param Podsumowanie symbolic := "Podsumowanie.csv";

#=====================================================================
param L_tygodni > 0, integer;
set Tygodnie := 1..L_tygodni;

param L_dni_na_tydzien > 0, integer;
param L_dni := L_tygodni * L_dni_na_tydzien;
set Dni := 1..L_dni;
param Tydzien {t in Dni union {0}}
	:= if t = 0 then 0 else (t-1) div L_dni_na_tydzien + 1;

param L_produktow > 0, integer;
set Produkty := 1..L_produktow;

#=====================================================================
param Czas_pracy_na_tydzien > 0;
param Czas_pracy_na_dzien
	:= Czas_pracy_na_tydzien / L_dni_na_tydzien;

param Zapas_pocz {Produkty} >= 0;
param Gotowosc_pocz {Produkty} binary;
check sum {j in Produkty} Gotowosc_pocz[j] = 1;

param Czas_wykonywania {Produkty} >= 0;
param Czas_przezbr {Produkty} >= 0;
param Koszt_przezbr {Produkty} >= 0;
param Jedn_koszt_zapasow {Produkty} >= 0;

param Popyt_w_tygodniu {Produkty, Tygodnie} >= 0;
param Popyt_w_dniu {j in Produkty, t in Dni} :=
	if (t mod L_dni_na_tydzien) = 0
		then Popyt_w_tygodniu[j, t/L_dni_na_tydzien] else 0;

#=====================================================================
var Gotowosc {Produkty, Dni union {0}} binary;
var Uruchomienie {Produkty, Dni} >= 0, <= 1;
var Produkcja {Produkty, Dni} >= 0;
var Zapas {Produkty, Dni union {0}} >= 0;

var Calk_koszt_zapasow >= 0;
var Calk_koszt_przezbr >= 0;
var Calk_L_przezbr >= 0;

#=====================================================================
minimize FC_2_5_a_Koszty_calk:
	Calk_koszt_przezbr + Calk_koszt_zapasow;

subject to

	Ogr_2_5_a_Calk_koszt_przezbr:
		Calk_koszt_przezbr =
			sum {j in Produkty, t in Dni}
				Koszt_przezbr[j] * Uruchomienie[j, t];

	Ogr_2_5_a_Calk_koszt_zapasow:
		Calk_koszt_zapasow =
			sum {j in Produkty, t in Tygodnie}
				Jedn_koszt_zapasow[j] * Zapas[j, t*L_dni_na_tydzien];

	Ogr_Calk_L_przezbr:
		Calk_L_przezbr =
			sum {j in Produkty, t in Dni}
				Uruchomienie[j, t];

	#--------------------------------------------------------------------------------------------------
	Ogr_Zapas_pocz {j in Produkty}:
		Zapas[j, 0] = Zapas_pocz[j];

	Ogr_Gotowosc_pocz {j in Produkty}:
		Gotowosc[j, 0] = Gotowosc_pocz[j];

	#--------------------------------------------------------------------------------------------------
	Ogr_2_5_b_Bilans_zapasow {j in Produkty, t in Dni}:
		Zapas[j, t-1] + Produkcja[j, t] = Popyt_w_dniu[j, t] + Zapas[j, t];

	Ogr_2_5_c_Produkcja_gdy_Gotowosc {t in Dni, j in Produkty}:
		Czas_wykonywania[j] * Produkcja[j, t] + Czas_przezbr[j] * Uruchomienie[j, t]
			<= Czas_pracy_na_dzien * (Gotowosc[j, t] + Gotowosc[j, t-1] );

	Ogr_2_5_d_Obciazenie_nie_wieksze_od_czasu_pracy {t in Dni}:
		sum {j in Produkty}
			(Czas_wykonywania[j] *Produkcja[j, t] + Czas_przezbr[j] * Uruchomienie[j, t])
		<= Czas_pracy_na_dzien;

	Ogr_2_5_e_Jednoznaczny_stan_maszyny {t in Dni}:
		sum {j in Produkty} Gotowosc[j, t] = 1;

	Ogr_2_5_f_Uruchomienie {j in Produkty, t in Dni}:
			Uruchomienie[j, t] >= Gotowosc[j, t] - Gotowosc[j, t-1];

	#--------------------------------------------------------------------------------------------------
	# Nierwnoci odcinajce
	#--------------------------------------------------------------------------------------------------
	NO_2_4_i_Gotowosc_po_uruchomieniu {j in Produkty, t in Dni}:
		Uruchomienie[j, t] <= Gotowosc[j, t];

	NO_2_4_j_Uruchomienie_jesli_nie_bylo_gotowosci {j in Produkty, t in Dni: t > 1}:
		Uruchomienie[j, t] <= 1 - Gotowosc[j, t-1];

	NO_2_5_j_Okres_wyczerpania_zapasu
			{j in Produkty, t in Dni, p in {0} union Dni: p <= L_dni-t}:
		Zapas[j, t-1] >= sum {s in t..t+p} Popyt_w_dniu[j, s]
			* (1 - Gotowosc[j, t-1] - sum {r in t..s} Uruchomienie[j, r]);

#=====================================================================

	display Tydzien, Czas_pracy_na_dzien, Popyt_w_dniu;

	solve;

	printf "Zmienna;Produkt;Dzie;Tydzie;Warto" > Wyniki;

	printf "\n%s;;;;%s", 'Model',Model >> Wyniki;
	printf "\n%s;;;;%s", 'Dane',Dane >> Wyniki;
	printf "\n%s;;;;%s", 'Data', time2str(gmtime(),'%F') >> Wyniki;
	printf "\n%s;;;;%s", 'Godzina', time2str(gmtime(),'%H:%M') >> Wyniki;
	printf "\n%s;;;;%d", 'L_tygodni', L_tygodni >> Wyniki;
	printf "\n%s;;;;%d", 'L_dni_na_tydzien', L_dni_na_tydzien >> Wyniki;
	printf "\n%s;;;;%d", 'L_produktow', L_produktow >> Wyniki;
	printf "\n%s;;;;%d", 'Czas_pracy', Czas_pracy_na_tydzien >> Wyniki;

	printf "\n%s;;;;%3.1f", 'Calk_koszt_zapasow', Calk_koszt_zapasow >> Wyniki;
	printf "\n%s;;;;%3.1f", 'Calk_koszt_przezbr', Calk_koszt_przezbr >> Wyniki;
	printf "\n%s;;;;%3.1f", 'Calk_L_przezbr', Calk_L_przezbr >> Wyniki;

	for {j in Produkty} {
		printf "\n%s;%d;;;%d", 'Zapas_pocz', j, Zapas_pocz[j] >> Wyniki;
		printf "\n%s;%d;;;%3.1f", 'Czas_wykonywania', j, Czas_wykonywania[j] >> Wyniki;
		printf "\n%s;%d;;;%3.1f", 'Koszt_przezbr', j, Koszt_przezbr[j] >> Wyniki;
		printf "\n%s;%d;;;%3.1f", 'Czas_przezbr', j, Czas_przezbr[j] >> Wyniki;
		printf "\n%s;%d;;;%3.1f", 'Jedn_koszt_zapasow', j, Jedn_koszt_zapasow[j] >> Wyniki;
	}

	for {j in Produkty, t in Dni} {
		printf "\n%s;%d;%d;%d;%3.1f", 'Popyt', j, t, Tydzien[t],  Popyt_w_dniu[j, t] >> Wyniki;
		printf "\n%s;%d;%d;%d;%3.1f", 'Produkcja', j, t, Tydzien[t],  Produkcja[j, t] >> Wyniki;
		printf "\n%s;%d;%d;%d;%d", 'Uruchomienie', j, t, Tydzien[t], Uruchomienie[j, t] >> Wyniki;
	}

	for {j in Produkty, t in Dni union {0}} {
		printf "\n%s;%d;%d;%d;%d", 'Gotowosc', j, t, Tydzien[t], Gotowosc[j, t] >> Wyniki;
		printf "\n%s;%d;%d;%d;%3.1f", 'Zapas', j, t, Tydzien[t], Zapas[j, t] >> Wyniki;
	}

	#--------------------------------------------------------------------------------------------------
	printf "%s;%s;%3.1f;%3.1f;%3.1f;%3.1f\n", Model, Dane,
		Calk_koszt_zapasow , Calk_koszt_przezbr, Calk_L_przezbr
		>> Podsumowanie ;

#=====================================================================
end;
